<?php

declare(strict_types=1);

namespace Drupal\search_api_exclude_lb\EventSubscriber;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\layout_builder\Event\SectionComponentBuildRenderArrayEvent;
use Drupal\layout_builder\LayoutBuilderEvents;
use Drupal\search_api_exclude_lb\SearchApiExcludeLbInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Removes render array during indexing if applicable.
 */
final class BlockComponentExclude implements EventSubscriberInterface {

  /**
   * Whether debug mode is enabled.
   *
   * @var bool
   */
  protected bool $debug;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user.
   * @param \Drupal\search_api_exclude_lb\SearchApiExcludeLbInterface $helper
   *   Our helper service.
   */
  public function __construct(
    protected AccountProxyInterface $currentUser,
    protected SearchApiExcludeLbInterface $helper,
  ) {
    $this->debug = $helper->getDebugMode();
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    // We want to run first, so we can prevent unnecessary processing too.
    $events[LayoutBuilderEvents::SECTION_COMPONENT_BUILD_RENDER_ARRAY] = [
      'onBuildRender',
      110,
    ];
    return $events;
  }

  /**
   * When it is a view block during indexing, return empty build.
   *
   * @param \Drupal\layout_builder\Event\SectionComponentBuildRenderArrayEvent $event
   *   The section component render event.
   */
  public function onBuildRender(SectionComponentBuildRenderArrayEvent $event): void {

    $definition = $event->getPlugin()->getPluginDefinition();
    $is_flagged = (
      // @todo Do we need to support
      // \Drupal\Component\Plugin\Definition\PluginDefinitionInterface as well?
      // It seems we always get arrays here.
      is_array($definition)
      && array_key_exists('search_api_exclude_lb', $definition)
      && $definition['search_api_exclude_lb'] === TRUE);

    if ($is_flagged === FALSE) {
      return;
    }

    $cacheableMetadata = new CacheableMetadata();
    $cacheableMetadata->addCacheTags($this->helper::getCacheTags());

    // Allow our processing only for indexing, or when debug mode is enabled and
    // the user has the permission to use/view it.
    $account = $this->currentUser->getAccount();
    if (property_exists($account, 'search_api_processor')) {
      if ($account->search_api_processor === 'rendered_item') {

        // Don't cache our manipulations for indexing.
        $cacheableMetadata->setCacheMaxAge(0);
        $event->addCacheableDependency($cacheableMetadata);

        $event->setBuild([]);
        // Abort any other processing in the render array we altered.
        $event->stopPropagation();
        return;
      }
    }

    if ($this->currentUser->hasPermission('use debug mode')) {

      $cacheableMetadata->addCacheContexts(['user.permissions']);
      $event->addCacheableDependency($cacheableMetadata);

      if ($this->debug === TRUE) {

        $label = $event->getPlugin()->getPluginId();
        if (is_array($definition) && array_key_exists('admin_label', $definition)) {
          $label = $definition['admin_label'];
        }

        $replacement = [
          '#type' => 'inline_template',
          '#template' => '<div style="padding: 1rem 2rem; background-color: beige">{{ "Skipped for indexing:"|t }} {{ label }}</div>',
          '#context' => [
            'label' => $label,
          ],
        ];
        $event->setBuild($replacement);
      }
    }
  }

}
