<?php

declare(strict_types=1);

namespace Drupal\search_api_exclude_lb\Form;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\State\StateInterface;
use Drupal\search_api_exclude_lb\SearchApiExcludeLb;
use Drupal\search_api_exclude_lb\SearchApiExcludeLbInterface;

/**
 * Defines the form to toggle debug mode.
 */
final class SettingsForm extends ConfigFormBase {

  /**
   * The State service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected StateInterface $state;

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'search_api_exclude_lb_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {

    $this->setState();
    $enabled = $this->state->get(SearchApiExcludeLbInterface::STATE_KEY, FALSE);

    $form['state'] = [
      '#type' => 'details',
      '#title' => $this->t('Debug mode'),
      '#description' => $this->t('Replaces all Layout Builder components skipped for indexing with debug info, for all users having permissions to use the debug mode'),
      '#open' => $enabled,
    ];

    if ($enabled === FALSE) {
      $form['state']['enable'] = [
        '#type' => 'submit',
        '#button_type' => 'secondary',
        '#value' => $this->t('Enable'),
        '#submit' => [[$this, 'enableDebugMode']],
        '#attributes' => [
          'class' => [
            'form-item',
          ],
        ],
      ];
    }
    else {
      $form['state']['disable'] = [
        '#type' => 'submit',
        '#button_type' => 'secondary',
        '#value' => $this->t('Disable'),
        '#submit' => [[$this, 'disableDebugMode']],
        '#attributes' => [
          'class' => [
            'form-item',
          ],
        ],
      ];
    }

    return $form;
  }

  /**
   * Disable debug mode.
   */
  public function disableDebugMode(): void {
    $this->setState();
    $this->state->set(SearchApiExcludeLbInterface::STATE_KEY, FALSE);
    $this->messenger()->addMessage($this->t('Debug mode disabled.'));
    $this->invalidateCacheTags();
  }

  /**
   * Enable debug mode.
   */
  public function enableDebugMode(): void {
    $this->setState();
    $this->state->set(SearchApiExcludeLbInterface::STATE_KEY, TRUE);
    $this->messenger()->addMessage($this->t('Debug mode enabled.'));
    $this->invalidateCacheTags();
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {}

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['search_api_exclude_lb.settings'];
  }

  /**
   * Invalidated render arrays that affect.
   */
  protected function invalidateCacheTags(): void {
    Cache::invalidateTags(SearchApiExcludeLb::getCacheTags());
  }

  /**
   * Set the State service.
   */
  protected function setState(): void {
    if (!isset($this->state)) {
      // @todo Use DI when we only have to support D11 and we don't have to deal
      // with the changed constructor parameters.
      // @phpstan-ignore-next-line
      $this->state = \Drupal::state();
    }
  }

}
